<?php
class Adapter_Sqlite {

	/**
	 * Constructor.
	 *
	 * @param string $dbName
	 */
	public function __construct($dbName) {
		$this->_dbName = $dbName;
	}

	/**
	 * Creates a connection to the database.
	 *
	 * @return bool if ok return true, else return false
	 */
	public function connect() {
		if ($this->_connection) {
			return true;
		}

		if (!function_exists('sqlite_open') || !file_exists($this->_dbName)) {
			return false;
		}

		$this->_connection = sqlite_open($this->_dbName);
		if (!is_resource($this->_connection)) {
			return false;
		}

		return true;
	}
	
	/**
	 * Closes an open SQLite database
	 *
	 * @return void
	 */
	public function close() {
		if (is_resource($this->_connection)) {
			sqlite_close($this->_connection);
			$this->_connection = null;
		}
	}

	/**
	 * Prepares and executes an SQL statement with bound data.
	 *
	 * @param string $sql
	 * @return mixed (resource|bool)
	 */
	function query($sql) {
		return sqlite_query($sql, $this->_connection);
	}

	/**
	 * Returns a list of the tables in the database.
	 *
	 * @return array
	 */
	public function listTables() {
		$sql = 'SELECT name FROM sqlite_master WHERE type=\'table\' and (name like \'%__message\' or name like \'%__topic\') '
			 . 'UNION ALL SELECT name FROM sqlite_temp_master '
			 . 'WHERE type=\'table\' and (name like \'%_message\' or name like \'%_topic\') ORDER BY name';

		return $this->_fetchAll($sql);
	}

	/**
	 * Returns the number of rows based on a WHERE clause.
	 *
	 * @param  mixed	$table The table to update.
	 * @param  mixed	$where SELECT COUNT(*) clause(s).
	 * @return int		The number of rows.
	 */
	public function getCount($table, $where = '') {
		$where = $this->_whereExpr($where);

		$sql = "SELECT COUNT(*) AS c FROM "
			 . $this->quoteIdentifier($table, true)
			 . (($where) ? " WHERE $where" : '');

		list($count) = $this->_fetchAll($sql);
		return $count['c'];
	}

	/**
	 * Deletes table rows based on a WHERE clause.
	 *
	 * @param  mixed		$table The table to update.
	 * @param  mixed		$where DELETE WHERE clause(s).
	 * @return int		  The number of affected rows.
	 */
	public function delete($table, $where = '') {
		$where = $this->_whereExpr($where);

		$sql = "DELETE FROM "
			 . $this->quoteIdentifier($table, true)
			 . (($where) ? " WHERE $where" : '');

		$this->query($sql);
		$changedRows = sqlite_changes($this->_connection);
		$this->query('VACUUM');
		return $changedRows;
	}

	/**
	 * Convert an array or string into a string to put in a WHERE clause.
	 *
	 * @param mixed $where
	 * @return string
	 */
	protected function _whereExpr($where) {
		if (empty($where)) {
			return $where;
		}
		if (!is_array($where)) {
			$where = array($where);
		}
		foreach ($where as &$term) {
			$term = '(' . $term . ')';
		}
		$where = implode(' AND ', $where);
		return $where;
	}

	/**
	 * Get rowset
	 *
	 * @param string $sql
	 * @return mixed if(ok) - array, else - false
	 */
	function _fetchAll($sql) {
		$rowset = array();
		$queryId = $this->query($sql);
		if (!$queryId) {
			return false;
		}
		while (($result = sqlite_fetch_array($queryId, SQLITE_ASSOC))) {
			$rowset[] = $result;
		}
		return $rowset;
	}

    /**
     * Quotes an identifier.
     *
     * @param string $ident
     * @return string
     */
	function quoteIdentifier($ident) {
        $ident = str_replace("'", "\'", $ident);
        return "'$ident'";
    }
}

$limit = 2;
$dbName = $argv['1'] ? $argv['1'] : '';

if (!file_exists($dbName)) {
    echo "The file '$dbName' does not exist.\nUsage: {$argv[0]} <db_name>\n";
	exit(1);
}

echo "Starting cleaning of the database '$dbName'\n";
$db = new Adapter_Sqlite($dbName);
$db->connect();
$tables = $db->listTables();
foreach($tables as $table) {
	$messagesNumber =  $db->getCount($table['name']);
	if ($messagesNumber > $limit) {
		echo "Cleaning the database table '{$table['name']}', which contains $messagesNumber records.\n";
		$deleted = $db->delete($table['name'], '1');
		echo "Processing of the table '{$table['name']}' completed. $deleted records were removed.\n";
	}
}

$db->close();

echo "Finished cleaning the database '$dbName'\n";
exit(0);
?>